//-------------------------------------------------------------------------
// Example of strategy based on 2 crossing SMA (c) Koshelev M.A.
//-------------------------------------------------------------------------
library RD4Strategy123;

uses
  SysUtils,  Classes,  StrategyInterfaceUnit,
  TechnicalFunctions in 'TechnicalFunctions.pas';

var
  // External parameters
  Currency: PChar = nil;
  TimeFrame: integer;
  LotSize: double;
  periodSMA: integer;    //    (  - )
  Shift: integer;        //    (  - Y)
  DayOfTraling: integer; //           
                         //    

    // custom variables
  OrderHandle: integer;
  OpenTime: TDateTime;
  IndSMA: integer;
  LastTime: TDateTime;
  info: TTradePosition;

{-----Init strategy---------------------------------------------------------}
procedure InitStrategy; stdcall;
begin
  StrategyShortName('RD4-123');
  StrategyDescription('Strategy based on RD4FX but simmetry');

  // Register external parameters
  RegOption('Currency', ot_Currency, Currency);
  ReplaceStr(Currency, 'EURUSD');

  RegOption('Timeframe', ot_Timeframe, TimeFrame);
  TimeFrame := PERIOD_D1;

  RegOption('LotSize', ot_Double, LotSize);
  SetOptionDigits('LotSize', 1);
  lotSize := 0.1;

  RegOption('SMA period', ot_Integer, periodSMA);
  SetOptionRange('SMA period', 10, MaxInt);
  periodSMA := 20;

  RegOption('Shift', ot_Integer, Shift);
  SetOptionRange('Shift', 3, 10);
  Shift := 3;

  RegOption('DayOfTraling', ot_Integer, DayOfTraling);
  SetOptionRange('DayOfTraling', 3, 10);
  DayOfTraling := 6;
end;

{-----Done strategy---------------------------------------------------------}
procedure DoneStrategy; stdcall;
begin
  FreeMem(Currency);
end;

{-----Reset strategy--------------------------------------------------------}
procedure ResetStrategy; stdcall;
var a: string;
begin
  a:=IntToStr(periodSMA)+';0'+';Simple (SMA)'+';Close';
  //   ,  ,    
  IndSMA := CreateIndicator(Currency, TimeFrame, 'MovingAverage', a);
  OrderHandle:=-1;
  LastTime:=0;
end;

{-----Process single tick---------------------------------------------------}
procedure GetSingleTick; stdcall;
var temp1: string;
    //temp2: double;
begin
  // check our currency
  if Symbol <> string(Currency) then exit;

  // set currency and timeframe
  SetCurrencyAndTimeframe(Symbol, TimeFrame);

  // check number of bars and SMA period
  if (Bars < (periodSMA+Shift+2)) then exit;

  //   
  if Time(0)<>LastTime then
     begin
       //      
       if GetOrderInfo(OrderHandle, info) and (TTradePositionType(info.PosType) = tp_Buy) then
          begin
            temp1:=FloatToStr(info.StopLoss);
            Breakpoint(1,temp1);
            //     -      DayOfTraling   ,
            //    DayOfTraling   ,    DayOfTraling+1  ,
            //          .
            if ((Time(0)-OpenTime) >= (DayOfTraling+1))
               and (Close(DayOfTraling+1) > Close(DayOfTraling+2))
               and (Close(DayOfTraling+1) > info.StopLoss)
               and (Close(DayOfTraling+1) < Close(1))
               then ModifyOrder(OrderHandle,0,Close(DayOfTraling+1),0);
            //      ,   .
            // . .       -
            //            Y  
            //      X+Y  
            if (GetIndicatorValue(IndSMA,1,0) < GetIndicatorValue(IndSMA,1+Shift,0))
               and (Close(1) > Close(1+Shift))
               and (Close(1) < Close(Shift+PeriodSMA+1))
               then CloseOrder(OrderHandle);
          end;

       //      (  )
       if GetOrderInfo(OrderHandle, info) and (TTradePositionType(info.PosType) = tp_Sell) then
          begin
            temp1:=FloatToStr(OrderStopLoss);
            Breakpoint(2, temp1);
            //     -      6   ,
            //    6   ,    7  ,
            //          .
            if ((Time(0)-OpenTime) >= (DayOfTraling+1))
               and (Close(DayOfTraling+1) < Close(DayOfTraling+2))
               and (Close(DayOfTraling+1) < info.StopLoss)
               and (Close(DayOfTraling+1) > Close(1))
               then ModifyOrder(OrderHandle,0,Close(DayOfTraling+1),0);
            //      ,   .
            // . .       -
            //            Y  
            //      X+Y  
            if (GetIndicatorValue(IndSMA,1,0) > GetIndicatorValue(IndSMA,1+Shift,0))
               and (Close(1) < Close(1+Shift))
               and (Close(1) > Close(Shift+PeriodSMA+1))
               then CloseOrder(OrderHandle);
          end;

       //        
       //if (OrderSelect(OrderHandle, SELECT_BY_TICKET, MODE_TRADES) = false)
       if (GetOrderInfo(OrderHandle, info) = false)
          and (GetIndicatorValue(IndSMA,1,0) > GetIndicatorValue(IndSMA,1+Shift,0))
          and (Close(1) < Close(1+Shift))
          and (Close(1) > Close(Shift+PeriodSMA+1)) then
          begin
            SendInstantOrder(Symbol,op_Buy,LotSize,Bid-(200*Point),0,'',0,OrderHandle);
            OpenTime:=Time(0);
          end;

       //        
       //if (OrderSelect(OrderHandle, SELECT_BY_TICKET, MODE_TRADES) = false)
       if (GetOrderInfo(OrderHandle, info) = false)
          and (GetIndicatorValue(IndSMA,1,0) < GetIndicatorValue(IndSMA,1+Shift,0))
          and (Close(1) > Close(1+Shift))
          and (Close(1) < Close(Shift+PeriodSMA+1)) then
          begin
            SendInstantOrder(Symbol,op_Sell,LotSize,Ask+(200*Point),0,'',0,OrderHandle);
            OpenTime:=Time(0);
          end;
     LastTime:=Time(0);
     end;
end;

exports

InitStrategy,
DoneStrategy,
ResetStrategy,
GetSingleTick;

end.